-------------------------------------------------------------------------------
--
-- Title       : Average_Filter
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : Algorithm of average filtering	for 3x3 matrix
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

-------------------------------------------------------------------------------

entity Average_Filter is 
	port(
		CLK 	  : in  std_logic;
		EN		  : in  std_logic;
		P_11 	  : in  std_logic_vector(7 downto 0);
		P_12 	  : in  std_logic_vector(7 downto 0);
		P_13 	  : in  std_logic_vector(7 downto 0);
		P_21 	  : in  std_logic_vector(7 downto 0);
		P_22 	  : in  std_logic_vector(7 downto 0);
		P_23 	  : in  std_logic_vector(7 downto 0);
		P_31 	  : in  std_logic_vector(7 downto 0);
		P_32 	  : in  std_logic_vector(7 downto 0);
		P_33 	  : in  std_logic_vector(7 downto 0);
		PIXEL_OUT : out std_logic_vector(7 downto 0)
		);
end Average_Filter;

-------------------------------------------------------------------------------

architecture Average_Filter of Average_Filter is
	
	-- auxiliary signals for first addition
	signal sum_reg_11 : std_logic_vector(8 downto 0);	
	signal sum_reg_12 : std_logic_vector(8 downto 0);
	signal sum_reg_13 : std_logic_vector(8 downto 0);   
	signal sum_reg_14 : std_logic_vector(8 downto 0);
	-- auxiliary signal for the last odd pixel in first addition
	signal aux_reg_11 : std_logic_vector(7 downto 0);
	
	-- auxiliary signals for second addition
	signal sum_reg_21 : std_logic_vector(9 downto 0);	
	signal sum_reg_22 : std_logic_vector(9 downto 0);
	-- auxiliary signal for the last odd pixel in second addition
	signal aux_reg_21 : std_logic_vector(7 downto 0);
	
	-- auxiliary signal for third addition
	signal sum_reg_31 : std_logic_vector(10 downto 0);
	-- auxiliary signal for the last odd pixel in third addition
	signal aux_reg_31 : std_logic_vector(7 downto 0);
	
	-- auxiliary signals for the last addition
	-- signal for the normalized sample - power of two
	signal sum_reg_41 : std_logic_vector(7 downto 0);
	-- signal for minimalization of error of normalization
	signal aux_reg_41 : std_logic_vector(5 downto 0);
	
	-- auxiliary signal for the final sample
	signal final_sample : std_logic_vector(7 downto 0);
	
	---------------------------------------------------------------------------
begin
	---------------------------------------------------------------------------
	
	First_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then 
				sum_reg_11 <= ('0' & P_11) + ('0' & P_12);	
				sum_reg_12 <= ('0' & P_13) + ('0' & P_21);
				sum_reg_13 <= ('0' & P_22) + ('0' & P_23);   
				sum_reg_14 <= ('0' & P_31) + ('0' & P_32);
				aux_reg_11 <= P_33;
			end if;
		end if;
	end process First_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Second_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				sum_reg_21 <= ('0' & sum_reg_11) + ('0' & sum_reg_12);	
				sum_reg_22 <= ('0' & sum_reg_13) + ('0' & sum_reg_14);
				aux_reg_21 <= aux_reg_11;
			end if;
		end if;
	end process Second_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Third_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				sum_reg_31 <= ('0' & sum_reg_21) + ('0' & sum_reg_22);
				aux_reg_31 <= aux_reg_21;
			end if;
		end if;
	end process Third_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Fourth_Add_and_Norm_Proc: process (CLK) is
		variable sum_temp : std_logic_vector(11 downto 0);
	begin
		if rising_edge(CLK) then
			if EN = '1' then 
				sum_temp := ('0' & sum_reg_31) + ("0000" & aux_reg_31);
				sum_reg_41 <= sum_temp(10 downto 3);
				aux_reg_41 <= sum_temp(11 downto 6);
			end if;
		end if;
	end process Fourth_Add_and_Norm_Proc;
	
	---------------------------------------------------------------------------
	
	Sub_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				final_sample <= sum_reg_41 - ("00" & aux_reg_41);
			end if;
		end if;
	end process Sub_Proc; 
	
	---------------------------------------------------------------------------
	
	PIXEL_OUT <= final_sample;
	
end Average_Filter;
