-------------------------------------------------------------------------------
--
-- Title       : DP_Ram_1xE
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : Dual-Port RAM with One Enable Controlling Both Ports
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

-------------------------------------------------------------------------------

entity DP_Ram_1xE is
	generic (
		constant DEPTH : integer := 8
		);
	port (
		clk : in std_logic;
		en : in std_logic;
		we : in std_logic;
		addra : in std_logic_vector(DEPTH-1 downto 0);
		addrb : in std_logic_vector(DEPTH-1 downto 0);
		di : in std_logic_vector(7 downto 0);
		doa : out std_logic_vector(7 downto 0);
		dob : out std_logic_vector(7 downto 0)
		);
end DP_Ram_1xE;

-------------------------------------------------------------------------------

architecture DP_Ram_1xE of DP_Ram_1xE is 
	
	-- type definition for the RAM
	type ram_type is array (2**DEPTH-1 downto 0) of std_logic_vector (7 downto 0);
	-- signal for RAM
	signal RAM : ram_type;	 
	-- auxiliary signals for addresses
	signal read_addra : std_logic_vector(DEPTH-1 downto 0);
	signal read_addrb : std_logic_vector(DEPTH-1 downto 0);  
	
begin  
	
	process (clk)
	begin
		if (clk'event and clk = '1') then
			if (en = '1') then
				if (we = '1') then
					RAM(conv_integer(addra)) <= di;
				end if;
				read_addra <= addra;
				read_addrb <= addrb;
			end if;
		end if;
	end process;
	
	doa <= RAM(conv_integer(read_addra));
	dob <= RAM(conv_integer(read_addrb));
	
end DP_Ram_1xE;
